from questions_three.scaffolds.common import precondition, skip  # noqa: F401
from questions_three.scaffolds.common.activate_reporters import \
    activate_reporters
from .execute_test_table import execute_test_table  # noqa: F401

activate_reporters()
